' *****************************************************************************
'
' Copyright. 2007. Symvionics, Inc. All rights reserved.
' 
' Symvionics Inc. ("Symvionics") grants a non-exclusive license for you to use
' this source code and the accompanying documentation, if any, free of charge,
' provided that this paragraph, and the following two paragraphs appear in all
' copies and in all derivative works.  In no event shall this source code or
' this license thereto impact upon or fall within the scope of any other
' agreement that you have with Symvionics.
' 
' BY USING THIS SOFTWARE, YOU FURTHER AGREE THAT, IN NO EVENT, SHALL SYMVIONICS
' BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR
' CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS
' SOFTWARE, THE ACCOMPANYING DOCUMENTATION, IF ANY, OR ANY DERIVATIVE WORKS.
' YOU AGREE TO DEFEND, INDEMNIFY, AND HOLD HARMLESS SYMVIONICS FROM AND AGAINST
' ALL FINES, PENALTIES, LIABILITIES, DAMAGES, COSTS, AND EXPENSES INCURRED
' BY SYMVIONICS AS A RESULT OF YOUR USE OF THIS SOFTWARE INCLUDING THIRD PARTY
' USE OF ANY DERIVATIVE WORKS CREATED BY YOU.
' 
' SYMVIONICS SPECIFICALLY DISCLAIMS ANY AND ALL WARRANTIES, INCLUDING, BUT NOT
' LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
' PARTICULAR PURPOSE.  THE SOFTWARE AND ACCOMPANYING DOCUMENTATION LICENSED
' HEREUNDER ARE PROVIDED "AS IS" WITHOUT WARRANTY OR OBLIGATION OF SUPPORT OR 
' MAINTENANCE.
'
' ****************************************************************************

' DataExport.vbs
' Exports data to CSV files based on data events triggered by a specified parameter.
' The exported files are named "DataExport_<n>.csv" where n begins at 1 and is 
' incremented for each file created.

On Error Resume Next

' Default output location for the exported data files. Change this to whatever 
' you like, or (optionally) you can pass it in as a command-line argument to the script.
' For example, from a command line: DataExport.vbs "C:\MyConfigFileDirectory"
gOutputFileDirectory = "C:\Configs"
gFileCount = 0

'Process the command line arguments for this script. All are optional
Set Args = WScript.Arguments
If Args.Count > 0 Then gOutputFileDirectory = Args(0)

' Add a backslash to the end of the output path if there isn't one already
If (Right(gOutputFileDirectory, 1) <> "\") Then
	gOutputFileDirectory = gOutputFileDirectory & "\"
End If

' Define an IADS Application Object and load the configuration file. This will allow us 
' to access the functions exposed through the IadsAutomation Interface so we can create 
' parameters, retrieve data, etc. This Application object will attempt to connect to a
' client that is already running.
Set Iads = CreateObject("Iads.Application")

'Initialize the parameters we wish to export. For this example I'm using 3 at random from 
'a config file that I tested this with, but you can change it to any number/names you like
Dim gParamsToExport(3)
Dim gParamNames(3)
gParamNames(0) = "C001_02"
gParamNames(1) = "C001_03"
gParamNames(2) = "IN001_10"

If (InitExportParams(gParamsToExport, gParamNames)) Then
	' Creates a FileSystemObject we can use to write data to CSV files
	Set gFSO = CreateObject("Scripting.FileSystemObject")
	Dim gFile ' File handle we'll use later on to write out data
	
	' This creates an IADS parameter in VB. This will fail if the specified parameter doesn't exist.
	' You can change this to any parameter you like in your own config.
	pName = "SquareWaveData"
	Set parameter = Iads.Configuration.CreateParameter( pName )
	If Err.Number <> 0 Then
		MsgBox "DataExport - Error, unable to load parameter: """ & pName & """ Aborting Script." 
		CleanUp()
		Set Iads = nothing
		WScript.Quit
	End If
	parameter.VBScriptDataCompatibility = True	' Need this after we create a param, otherwise 
												' we won't be able to access the data here in VB
												
	Set currTime = CreateObject("IadsTime.IadsTime")
	
	' Start and end times of each export
	Set startTime = CreateObject("IadsTime.IadsTime")
	Set endTime = CreateObject("IadsTime.IadsTime")
		
	' This gets the total number of data points for the parameter
	NumPts = parameter.GetNumDataPoints(parameter.StartTime, parameter.EndTime)
	TimeIncr = (parameter.EndTime.TotalSeconds - parameter.StartTime.TotalSeconds) / NumPts
	exportingData = False
	
	' Set the current time of the parameter to the very beginning. This ensures that when we start
	' getting values, we'll start at the beginning of the array of data points
	parameter.CurrentTime = parameter.StartTime
	currTime.SetTime(parameter.StartTime)
	
	' Value() retrieves a data point for this parameter. Passing a 1 into the call to Value() here
	' will advance the internal pointer of the parameter so that subsequent calls to Value() will
	' get the next value in the array of data points.
	Value1 = parameter.Value(1)
	For dpIndex = 1 To NumPts-1	
		If (exportingData = False) Then
			If (checkStartTrigger(Value1)) Then
				exportingData = True
				startTime.SetTime(currTime)
			End If
	    Else
	    	If (checkEndTrigger(Value1)) Then
	    		exportingData = False
	    		endTime.SetTime(currTime)
	    		PerformDataExport()
	    	End If
		End If

		currTime.SetTime(parameter.CurrentTime)
		Value1 = parameter.Value(1)
	Next

	' Close the data file if we we're still exporting
	If (exportingData = True) Then
		endTime.SetTime(currTime)
		PerformDataExport()
	End If
	
	MsgBox(gFileCount & " data files exported successfully!")
	
	CleanUp()
Else
	MsgBox("Error initializing parameters for data export! Please verify that the names are typed correctly and they exist in your config file.")
End If

Set Iads = Nothing
' Done!!

' This function initializes the parameters that will be used to export data to a CSV file.
' It returns True if all parameters are imported successfully, False if there are any errors.
Function InitExportParams(byref params, byref names)
	InitSuccess = True
	If (UBound(params) = UBound(names)) Then
		For index = 0 To UBound(names)-1
			' This creates an IADS parameter in VB. This will fail if the specified parameter doesn't exist.
			Set params(index) = Iads.Configuration.CreateParameter( names(index) )

			' Make sure the parameter was created properly			
			If Err.Number<> 0 Then
				InitSuccess = False
			End If

			params(index).VBScriptDataCompatibility = True
		Next
		
		InitExportParams = InitSuccess
	Else
		InitExportParams = False
	End If
End Function

' This function checks for the data event that triggers the file export
' In this example it is defined as a data value being equal to 1.0
Function checkStartTrigger(Val1)
	checkStartTrigger = (Val1 = 1.0)
End Function

' This function checks for the data event that triggers the file export
' In this example it is defined as a data value being equal to 0.0
Function checkEndTrigger(Val1)
	checkEndTrigger = (Val1 = 0.0)
End Function

' Performs a data export based on the start and end times gathered from the triggers
Sub PerformDataExport()
	gFileCount = gFileCount + 1
	
	'Open the file to write to
	fName = "DataFile_" & gFileCount & ".csv"
	WScript.Echo("Beginning export of data to " & gOutputFileDirectory & fName)
	Set gFile = gFSO.OpenTextFile(gOutputFileDirectory & fName, 2, True)
	
	' Write out the file header (parameter names) and get the highest parameter sample rate
	SampleRate = -1.0
	gFile.Write("TIME")
	For i = 0 To UBound(gParamNames)-1
		sr = gParamsToExport(i).SampleRate
		If (SampleRate < sr) Then
			SampleRate = sr
		End If
		
		gFile.Write("," & gParamNames(i))
	Next
	gFile.WriteLine()

	startTimeInSeconds = startTime.TotalSeconds
	
	' Set the sample rate on all of the params to the highest sample rate so we "square off" 
	' all the data and get a matching number of data values for all of the parameters
	WScript.Echo("Setting current time to start time: " & startTime.TimeString)
	For j = 0 To UBound(gParamsToExport)-1
		gParamsToExport(j).InterpolatedSampleRate = SampleRate
		paramStart = gParamsToExport(j).StartTime.TotalSeconds
		If (paramStart <= startTimeInSeconds) Then
			gParamsToExport(j).CurrentTime = startTime
			WScript.Echo("Start time set successfully for " & gParamsToExport(j).Name)
		Else
			WScript.Echo("No data found at start time for " & gParamsToExport(j).Name & ", resetting to its start time")

			gParamsToExport(j).CurrentTime = gParamsToExport(j).StartTime
			WScript.Echo("Start time successfully reset for " & gParamsToExport(j).Name)
		End If
	Next
		
	Set thisTime = CreateObject("IadsTime.IadsTime")
	thisTime.SetTime(startTime)
	
	WScript.Echo("Writing out data, row-by-row...")
	' Write out the data, row by row
	Do While thisTime.TotalSeconds < endTime.TotalSeconds
		gFile.Write(thisTime.TimeString)
		For paramIndex = 0 To UBound(gParamsToExport)-1
			paramTime = gParamsToExport(paramIndex).StartTime.TotalSeconds
			cTime = thisTime.TotalSeconds
			If (paramTime <= cTime) Then
				gFile.Write("," & gParamsToExport(paramIndex).Value(1))
			Else
				gFile.Write(", ")
			End If
		Next
		thisTime.AdvanceTime 1, SampleRate
		gFile.WriteLine()
	Loop
	
	WScript.Echo("Data written to file, closing...")
	Set thisTime = Nothing
	
	'Close the export file
	gFile.Close()
	Set gFile = Nothing
	
	WScript.Echo("Successfully exported data to " & gOutputFileDirectory & fName)
	WScript.Echo("")
End Sub

' Cleans up the objects we created
Sub CleanUp()
	Set gFSO = Nothing
	Set config = Nothing
	Set parameter = Nothing
	
	For index = 0 To UBound(gParamsToExport)-1
		Set gParamsToExport(index) = Nothing
	Next
End Sub
